@@ -31,6 +31,7 @@ from Products.CMFCore.utils import getToolByName
 
 import zLOG
 from config import SEPARATOR, PROPS, ALLPROPS, NONATPROPS, TYPEMAP, DEFAULT_ENCODING
+from config import BRAIN_METHODS
 try:
     from App.special_dtml import DTMLFile
 except:
@@ -453,14 +454,17 @@ class ContentExporter(UniqueObject, SimpleItem):
                 'groups': group_info,
                }
 
-    def write_folder(self, folder, path):
-        """ Write the contents of folder out """
+    def write_folder(self, folder, path, extras={}):
+        """ Write the contents of folder out 
+            meta is for extra metadata for the contained objects
+            e.g. {obj_id:{meta:'foobar'}, }
+        """
         folderdoc = self.get_doc(delimiter=',')
         path = path[self.lenportal:]
 
         for id in folder.objectIds():
             if not id.startswith('.'):
-                obj = getattr(folder,id,None)
+                obj = getattr(folder, id, None)
                 # getTypeInfo can return Folder for python scripts so check
                 if obj and not str(obj) == '<PythonScript at %s>' % id:
                     try:
@@ -470,7 +474,8 @@ class ContentExporter(UniqueObject, SimpleItem):
                         objtype = None
                     if objtype:
                         doc = self.get_doc()
-                        self.export_object(id, obj, objtype, path, doc)
+                        self.export_object(id, obj, objtype, path, doc, 
+                                           extras.get(id, {}))
                         self.write_folder_element(id=id, objtype=objtype,
                                                   doc=folderdoc)
         output = self.get_output(folderdoc)
@@ -487,8 +492,8 @@ class ContentExporter(UniqueObject, SimpleItem):
         if format == 'XML':
             self.dom = getDOMImplementation()
         self.workflow_tool = getToolByName(portal, 'portal_workflow')
-        portalname = portal.getId()
-        self.lenportal = len(portalname) + 1
+        self.portalname = portal.getId()
+        self.lenportal = len(self.portalname) + 1
         self.out = ['Log started at %s' % DateTime()]
         # Give this a request attribute since some methods expect it
         # and it is not available when used via current plone 
@@ -511,8 +516,8 @@ class ContentExporter(UniqueObject, SimpleItem):
 
         if not root.startswith('/'):
             root = '/%s' % root
-        if len(root) > 1 and not root.startswith('/%s/' % portalname):            
-            root = '/%s%s' % (portalname, root)            
+        if len(root) > 1 and not root.startswith('/%s/' % self.portalname):            
+            root = '/%s%s' % (self.portalname, root)            
         catalog = getToolByName(portal, 'portal_catalog')
         # Just return everything then filter for folderish later
         results = catalog(path={'query': root})
@@ -551,8 +556,11 @@ class ContentExporter(UniqueObject, SimpleItem):
         return self.out
 
 
-    def export_object(self, id, obj, objtype, path, doc):
-        """ export file content to filesystem """
+    def export_object(self, id, obj, objtype, path, doc, extra={}):
+        """ export file content to filesystem 
+            extra is for extra metadata for the contained objects
+            e.g. {meta:'foobar', humbug:'mint'}
+        """
         modified = self.properties_marshall(obj, doc)
         # do required fields
         for prop, data in PROPS['fixed'].items():
@@ -567,7 +575,13 @@ class ContentExporter(UniqueObject, SimpleItem):
                                data=data,
                                doc=doc)
         self.workflow(obj, doc)
+        # sort out metadata
         self.at_marshall(obj, path, doc)
+        # Do any extra metadata
+        for key, value in extra.items():
+            self.write_element(name=key.lower(),
+                               data=value,
+                               doc=doc)
 
         if hasattr(obj,'data'):
             treatas = 'binary'
@@ -586,6 +600,47 @@ class ContentExporter(UniqueObject, SimpleItem):
                     value = str(value)
                 output = self.get_output(doc, value.replace('\r', ""))
                 self.write_file(path, id, output, modified)
+            # Handle webpage types which are not isPrincipiaFolderish
+            # but are really folders that contain hidden extra items
+            if hasattr(obj, 'objectIds'):
+                try:
+                    num_objs = len(obj.objectIds())
+                except:
+                    num_objs = 0
+                if num_objs:
+                    if BRAIN_METHODS:
+                        # Add extra metadata from contained brains
+                        meta = {}
+                        for obj_id in obj.objectIds():
+                            try:
+                                subobj = obj[obj_id]
+                            except:
+                                subobj = None
+                        if subobj:
+                            for bmethod in BRAIN_METHODS.keys():
+                                battribs = BRAIN_METHODS[bmethod]
+                                try:
+                                    brains = getattr(obj, bmethod)()
+                                except:
+                                    brains = []
+                                for brain in brains:
+                                    extras = {}
+                                    for battr in battribs:
+                                        try:
+                                            exvalue = getattr(brain, battr)()
+                                            if value:
+                                                extras[battr] = exvalue
+                                        except:
+                                            pass
+                                    if extras:
+                                        obj_id = brain.getObject().getId()
+                                        if meta.has_key(obj_id):
+                                            meta[obj_id].update(extras)
+                                        else:
+                                            meta[obj_id] = extras
+                    self.out.append('Writing files: %s has %s hidden objects' % (id, num_objs))
+                    filepath = '/%s/%s/%s.content' % (self.portalname, path, id)
+                    self.write_folder(obj, filepath, meta)
         else:
             if treatas == 'folder':
                 self.write_file(os.path.join(path, id), '.properties',
