@@ -59,6 +59,9 @@ NONATPROPS = {'Calendar Item':{'start_date':'startDate',
               'Link':{'remote_url':'remoteUrl'}     
               }
 
+# Extra custom Product's methods for hidden contained objects metadata
+BRAIN_METHODS = {}
+
 #Used to encode content written to text files,
 #when characters are not within the ascii character range
 DEFAULT_ENCODING='utf8'
